from __future__ import annotations

from pathlib import Path
from re import compile
from shutil import move
from tempfile import gettempdir
from zipfile import ZipFile

from requests import get

REPO = compile(
    r"(?P<author>[a-zA-Z0-9_-]+)\/(?P<repo>[a-zA-Z0-9_-]+)(@(?P<branch>[a-zA-Z0-9_-]+))?"
)


def clone_github(repo: str, dest: Path) -> None:
    match = REPO.search(repo)

    if not match:
        raise ValueError(f"Invalid repo: {repo}")

    url = (
        f"https://github.com/{match.group('author')}/{match.group('repo')}"
        f"/archive/refs/heads/{match.group('branch') or 'master'}.zip"
    )

    temp_path = Path(gettempdir()) / f"{hash(url)}.zip"
    temp_path.write_bytes(get(url).content)

    with ZipFile(temp_path) as zip_file:
        print("Cloned to ", dest.absolute())
        zip_file.extractall(dest.parent)

    move(
        dest.parent / f"{match.group('repo')}-{match.group('branch') or 'master'}", dest
    )



from __future__ import annotations

from json import dumps, loads
from pathlib import Path
from typing import Optional


class Registry:
    def __init__(self, path: Optional[Path] = None) -> None:
        self.path = path or Path.home() / ".config" / "noo"
        self.path.mkdir(parents=True, exist_ok=True)

        self.value = self.read(cache=False)

    def read(self, cache: bool = True) -> dict[str, str]:
        path = self.path / "registry.json"

        if not path.exists():
            return {}

        data = loads(path.read_text())

        if cache:
            self.value = data

        return data

    def write(self) -> None:
        path = self.path / "registry.json"

        if not path.exists():
            path.touch()

        path.write_text(dumps(self.value, indent=2))

    def set_item(self, key: str, ref: str | Path) -> None:
        if isinstance(ref, Path):
            absolute_ref = "file:" + str(ref.absolute())
        else:
            absolute_ref = ref

        self.value[key] = absolute_ref

        self.write()

    def get_item(self, key: str) -> str | Path:
        ref = self.value[key]

        if ref.startswith("file:"):
            return Path(ref[5:])

        return ref

    def del_item(self, key: str) -> None:
        del self.value[key]



from __future__ import annotations

from datetime import datetime

from ..models import ReadVariable


def get_variables(name: str) -> dict[str, dict[str, str | int]]:
    data = {}

    now = datetime.now()

    data["var"] = {}
    data["noo"] = {
        "year": now.year,
        "month": now.month,
        "day": now.day,
        "hour": now.hour,
        "minute": now.minute,
        "second": now.second,
        "name": name,
    }

    return data


def read_variables(variables: list[ReadVariable]) -> dict[str, str | int]:
    data = {}

    for variable in variables:
        extra = ""
        if variable.default is not None:
            extra = f" [{variable.default}]"

        value = input((variable.prompt or f"Enter {variable.name}") + f"{extra}: ")

        data[variable.name] = value if value else variable.default

    return data



from typing import Optional

from pydantic import BaseModel

from .actions import ActionType, List


class ReadVariable(BaseModel):
    name: str
    prompt: Optional[str] = None
    default: Optional[str] = None


class Step(BaseModel):
    name: str
    actions: List[ActionType] = []


class Noofile(BaseModel):
    name: Optional[str] = None
    remote: str
    read: List[ReadVariable] = []
    steps: List[Step] = []



from __future__ import annotations

from pathlib import Path

from typer import echo

from ..resolvers import clone_github, clone_local
from .resolver import Resolver
from .runner import Runner
from .variables import get_variables, read_variables


class NooCore:
    def __init__(self) -> None:
        self.resolver = Resolver()

    def clone(self, name: str, noofile: str, dest: Path) -> None:
        spec = self.resolver.resolve(noofile)

        echo(f"Starting clone process for {spec.name or name}.")

        if spec.remote.startswith("git:"):
            clone_github(spec.remote[4:], dest)
        elif spec.remote.startswith("file:"):
            clone_local(Path(spec.remote[5:]), dest)
        else:
            raise ValueError(f"Invalid remote: {spec.remote}")

        variables = get_variables(name)
        variables["var"].update(read_variables(spec.read))

        runner = Runner(dest, spec.steps, variables)
        runner.run()



from __future__ import annotations

from json import loads
from pathlib import Path

from requests import get
from yaml import safe_load

from ..models import Noofile


class Resolver:
    def __init__(self) -> None:
        pass

    def _resolve_local(self, file: str) -> Noofile:
        path = Path(file)

        if not path.exists():
            raise ValueError(f"No such file: {path}")

        return Noofile(**safe_load(path.read_text()))

    def _resolve_http(self, path: str) -> Noofile:
        data = get(path)

        if data.status_code != 200:
            raise ValueError(f"Invalid response looking up noofile: {data.status_code}")

        return Noofile(**safe_load(data.content))

    def resolve(self, loc: str) -> Noofile:
        if loc.startswith("http"):
            return self._resolve_http(loc)

        return self._resolve_local(loc)



from .actions import CreateAction, DeleteAction, RenameAction, ReplaceAction
from .noofile import Noofile, ReadVariable, Step

__all__ = (
    "CreateAction",
    "DeleteAction",
    "Noofile",
    "ReadVariable",
    "RenameAction",
    "ReplaceAction",
    "Step",
)



from typing import List, Literal, Optional, Union

from pydantic import BaseModel


class Action(BaseModel):
    action: str


class ReplaceAction(Action):
    action: Literal["replace"]
    files: List[str]
    src: str
    dest: str


class DeleteAction(Action):
    action: Literal["delete"]
    files: List[str]


class CreateAction(Action):
    action: Literal["create"]
    file: str
    content: Optional[str] = None


class RenameAction(Action):
    action: Literal["rename"]
    file: str
    dest: str


ActionType = Union[ReplaceAction, DeleteAction, CreateAction, RenameAction, Action]
